# -*- coding:utf-8 -*-
"""
jax-encrypt cmd, provide encryption and decryption functions
"""
import sys
import os
import argparse
# sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from jax_tools.encrypt import AESCipher
from jax_tools.encrypt import get_local_key


# Ensure the existence of the encryption key file
JAX_ENCRYPT = AESCipher().encrypt
JAX_DECRYPT = AESCipher().decrypt


def main() -> None:
    """
    main
    Returns:

    """
    parser = argparse.ArgumentParser(
        description="Jax-encrypt is used to encrypt and decrypt your sensitive information", prog='JAX-ENCRYPT')
    parser.add_argument('-d', '--decrypt', type=str, default=None, help='Specifies the string to be decrypted')
    parser.add_argument('-e', '--encrypt', type=str, default=None, help='SSpecifies the encryption string')
    parser.add_argument('-k', '--key', type=str, default=None, help='Specifies the encryption key')
    parser.add_argument('-t', '--type', type=str, default='gcm', help='Specifies the encryption type, cbc or gcm')
    parser.add_argument('-v', '--version', action='version', version='1.0')
    options = parser.parse_args()
    if options.key is not None:
        key = options.key
    else:
        key = get_local_key()
    if options.type.lower().__contains__('cbc'):
        cbc_mode = True
    else:
        cbc_mode = False
    if options.decrypt is not None:
        print(AESCipher(key, cbc_mode).decrypt(options.decrypt))
    if options.encrypt is not None:
        print(AESCipher(key, cbc_mode).encrypt(options.encrypt))


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""
Command tool
"""
import os
import shlex
import subprocess


class Command(object):
    """
    Command tool
    """
    @staticmethod
    def exec_safety(cmd: str) -> None:
        """
        Execute command use safety method
        Args:
            cmd (str): command
        Returns:
            None
        """
        subprocess.call(shlex.split(cmd))

    @staticmethod
    def get_output_safety(cmd: str) -> str:
        """
        Get command output use safety method
        Args:
            cmd (str): command
        Returns:
            str: command output
        """
        return subprocess.check_output(shlex.split(cmd)).decode('utf-8')

    @staticmethod
    def exec_without_safety(cmd: str) -> None:
        """
        Execute command without safety method, suggest you validate the command before use this method
        Please Ensure the command is safe
        Args:
            cmd (str): command
        Returns:
            None
        """
        os.system(cmd)

    @staticmethod
    def get_output_without_safety(cmd: str) -> str:
        """
        Get command output without safety method, suggest you validate the command before use this method
        Please Ensure the command is safe
        Args:
            cmd (str): command
        Returns:
            str: command output
        """
        return os.popen(cmd).read()



# -*- coding: utf-8 -*-
"""
Upgrade local pip package to the latest version.
"""
import os


def upgrade() -> None:
    """
    Main function
    Returns:

    """
    # check current directory have dist folder
    if not os.path.exists('dist'):
        print('No dist folder found')
        return
    package_name = os.listdir('dist')[0]
    os.system(f'pip install dist/{package_name} --upgrade')


def main() -> None:
    """
    Main function
    Returns:

    """
    upgrade()



# -*- coding: utf-8 -*-
"""
Build Jax Tools
"""
import subprocess
import sys


def update_version() -> None:
    """
    Update version
    Returns:

    """
    setup_py = 'setup.py'
    last_version = open(setup_py, 'r', encoding='utf-8').read().split('version="')[1].split('"')[0]
    last_num = int(last_version.split('.')[-1])
    new_version = '.'.join(last_version.split('.')[:-1]) + '.' + str(last_num + 1)
    with open(setup_py, 'r', encoding='utf-8') as f:
        setup_file = f.read()
    setup_file = setup_file.replace('version="{}"'.format(last_version), 'version="{}"'.format(new_version))
    with open(setup_py, 'w', encoding='utf-8') as f:
        f.write(setup_file)

    print('Update Version to {}'.format(new_version))


def build() -> None:
    """
    Build Jax Tools
    Returns:

    """
    # Delete dist folder
    subprocess.call('rm -rf dist', shell=True)
    # Build
    subprocess.call('python setup.py sdist', shell=True)


def main() -> None:
    """
    Main function
    Returns:

    """
    try:
        if not sys.argv[1]:
            print("need update version")
            update_version()
    except IndexError:
        update_version()
    finally:
        build()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""
Set proxies for requests
"""
from jax_tools.network_test import check_connectivity
from jax_tools.utils.base import KT, VT
from typing import Optional


def local_proxies(host: str = '127.0.0.1', port: int = 19999, protocol: str = 'http') -> Optional[dict[KT, VT]]:
    """
    Set proxies for requests
    Args:
        host (str): host for proxy
        port (int): port for proxy, defaults to 19999
        protocol (str): protocol for proxy, defaults to http

    Returns:
        proxies or None
    """
    enable_proxy = True
    if not enable_proxy:
        return None
    http_proxy_address = host
    http_proxy_port = port
    proxy_address = '{}://{}:{}'.format(protocol, http_proxy_address, http_proxy_port)
    if check_connectivity(http_proxy_address, http_proxy_port):
        proxies = {
            'http': proxy_address,
            'https': proxy_address,
        }
    else:
        proxies = None
    return proxies



# -*- coding: utf-8 -*-
"""
Check if the target IP and port is open
"""
import socket
import sys
from ping3 import ping
from jax_tools.colorful_font import Green
from jax_tools.colorful_font import Red
from jax_tools.colorful_font import Yellow
from jax_tools.colorful_font import ColorfulFont
from typing import Optional, Union


class PrintMSG(object):
    """
    Print message
    """
    need_print = True

    def __init__(self, msg: Union[str, ColorfulFont] = str()) -> None:
        """
        Init
        Args:
            msg : Message to print
        """
        if self.need_print:
            print(msg)


def check_icmp_connectivity(ip: str) -> bool:
    """
    Check if the target IP is reachable
    Args:
        ip(str): IP address or hostname

    Returns:
        bool: True if the target IP is reachable, False otherwise
    """
    try:
        # Use ping3 to check if the target IP is reachable
        if ping(ip):
            return True
        else:
            return False
    except Exception as e:
        print(e)
        return False


def check_connectivity(ip: str, port: Optional[int], need_print_msg: Optional[bool] = False) -> bool:
    """
    检查目标IP和端口是否可以连接
    Args:
        ip (str): Destination IP address or hostname
        port (Optional[init]): Destination port
        need_print_msg (bool): Whether to print the result

    Returns:
        bool: True if the port is open, False if the port is closed or the connection is refused
    """
    print_msg = PrintMSG
    print_msg.need_print = need_print_msg
    # 如果没有端口，则只检查IP是否可以ping通
    if not port:
        if check_icmp_connectivity(ip):
            print_msg(Green.bold("Connection success"))
            return True
        else:
            print_msg(Red("Connection error: ping not work"))
            return False
    s = None
    network_unreachable = 'Network is unreachable'
    result_dict = {
        35: 'The port is closed',
        61: 'Connection refused, firewall restrictions may be set on the target port',
        111: 'Connection refused',
        113: 'No route to host',
        101: network_unreachable,
        110: 'Connection timed out',
        10035: network_unreachable,
        10061: 'Connection refused',
        10060: 'Connection timed out',
        10051: network_unreachable,
        10065: 'No route to host',
        10064: 'Host is down',
        10013: 'Permission denied',
        10049: 'Can\'t assign requested address'
    }
    try:
        # 创建一个socket对象
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.settimeout(5)  # 设置超时时间（单位：秒）
        verify_port(port, print_msg)
        # 尝试连接到目标IP和端口
        result = s.connect_ex((ip, port))
        if result == 0:
            print_msg(Green.bold("Connection success"))
            result = True
        elif result in result_dict.keys():
            if check_icmp_connectivity(ip):
                print_msg(Red("Connection error: " + result_dict[result]))
            else:
                print_msg(Red("Connection error: " + result_dict[result] + "and ping not work"))
            result = False
        else:
            print_msg(Red("Connection error：" + str(result)))
            result = False
    except socket.error as e:
        print_msg(Red("Connection error：" + e))
        result = False
    finally:
        if s:
            s.close()
    return result


def verify_port(port: Union[str, int], print_msg: callable) -> None:
    """
    Verify if the port is valid
    Args:
        port (str): Port number
        print_msg (callable): Print message object

    Returns:
        bool: True if the port is valid, False otherwise
    """
    if isinstance(port, str):
        try:
            port = int(port)
            if port < 0 or port > 65535:
                print_msg(Yellow("The port must be between 0 and 65535"))
                sys.exit(1)
        except ValueError:
            print_msg(Yellow("The port must be a number"))
            sys.exit(1)




# -*- coding: utf-8 -*-
"""
Install python3
"""
import subprocess, os
import sys


def install_python(version='3.9.13'):
    """
    Install python3.9.13
    Args:
        version (str): version

    Returns:

    """
    if os.path.exists('/usr/bin/python3'):
        print('python3 is exist already. we will continue to install python{0} to /usr/local/python{0}'.format(version))
    subprocess.call('yum install gcc zlib zlib-devel libffi-devel openssl-devel -y', shell=True)
    os.chdir('/tmp')
    if not os.path.exists('/tmp/python{}.tar.xz'.format(version)):
        print('Downloading python{}.tar.xz'.format(version))
        subprocess.call('curl -fsSL https://www.python.org/ftp/python/{}/Python-{}.tar.xz > python{}.tar.xz'.format(
            version, version, version), shell=True)
    subprocess.call('tar xf python{}.tar.xz -C /usr/local/src/'.format(version), shell=True)
    os.chdir('/usr/local/src/Python-{}/'.format(version))
    subprocess.call('./configure --prefix=/usr/local/python{}'.format(version), shell=True)
    subprocess.call('make', shell=True)
    subprocess.call('make install', shell=True)
    subprocess.call("sed -i '210,212s/#//' /usr/local/src/Python-{}/Modules/Setup".format(version), shell=True)
    subprocess.call("sed -i '205s/#//' /usr/local/src/Python-{}/Modules/Setup".format(version), shell=True)
    subprocess.call('make', shell=True)
    subprocess.call('make install', shell=True)
    print('python{} has been installed successfully'.format(version))
    print('You can find python{0} at /usr/local/python{0}/bin/}'.format(version))
    if os.path.exists('/usr/bin/python3'):
        print('/usr/bin/python3 is exist already. if you want to replace it, you can use command as this: '
              'mv /usr/bin/python3 /usr/bin/python3_bakup;ln -s /usr/local/python{0}/bin/python3 /usr/bin/'.format(version))
    print('Current python3 version is: ')
    subprocess.call('python3 --version', shell=True)


def main():
    """
    Main function
    Returns:

    """
    try:
        version = sys.argv[1]
        if not int(version.replace('.', '')):
            print('Invalid version number')
            exit()
    except IndexError:
        version = '3.9.13'
    except ValueError:
        print('Invalid version number')
        exit()
    install_python(version)



# -*- coding: utf-8 -*-
"""
utils
"""
import os
ROOT_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
HOME_DIR = os.path.expanduser('~')
JAX_DATA_DIR = os.path.join(HOME_DIR, '.jax')
JAX_KEY_FILE = os.path.join(JAX_DATA_DIR, '.jax_key')



"""
Jax cmd
"""


def main() -> None:
    """
    provide jax cmd for print help info
    Returns:

    """
    print("JAX Tools Commands:")
    print("nd: Check network connectivity")
    print("jax-encrypt: Encrypt and decrypt sensitive information")



#!env python3
# -*- coding:utf-8 -*-
"""
Poppy Entrypoint
"""
from jax_tools.cmd_tools.pam import PasswordManager as pm
import sys
import getpass
import subprocess
import threading
try:
    from win10toast import ToastNotifier
except ImportError:
    ToastNotifier = None


def print_help() -> None:
    """
    Print Help message
    Returns:

    """
    print(u'--help 打印帮助信息')
    print(u'--list 打印账号列表')
    print(u'--delete 删除账号信息,指定账号名,如--delete account1')
    print(u'--add 添加账号信息,指定账号名')
    print(u' --get 通过NUM或账号名获取密码如 --get 1或-g 1')
    print(u'输入非上述的--开头的内容，则表示搜索账号，并将搜索到的账号的密码复制到剪贴板')


def minimize_window() -> None:
    """
    Minimize window
    Returns:

    """
    applescript = """
    tell application "System Events"
        keystroke "m" using command down
    end tell
    """
    try:
        subprocess.call(["osascript", "-e", applescript])
    except FileNotFoundError:
        pass


def update_account() -> None:
    """
    Update account
    Returns:

    """
    account_count = pm.print_account_list()
    num = int(input(u'请输入您想要更新的账号NUM(1~{} 从上方首列获取)：'.format(account_count)))
    account_info = pm.get_user_info(num)
    name = account_info.get(pm.NAME_KEY)
    username = account_info.get(pm.USERNAME_KEY)
    new_name = input(u'请输入新的账号名称(默认名称:{})：'.format(name))
    new_url = input(u'请输入新的url(默认url:{}): '.format(account_info.get(pm.URL_KEY)))
    new_username = input(u'请输入新的用户名(默认用户名{}): '.format(username))
    if not new_name:
        new_name = name
    if not new_username:
        new_username = username
    password = getpass.getpass(u'请输入登录密码或验证token: ')
    if pm.delete_account(name, print_msg=False):
        pm.add_account(new_name, new_url, new_username, password, num, print_msg=False)
        print(u'NUM:{}, name:{}更新成功'.format(num, name))


def delete_account() -> None:
    """
    Delete account
    Returns:

    """
    if len(sys.argv) > 2:
        name = ''.join(sys.argv[2:])
        pm.delete_account(name)
    else:
        name = input(u'请输入您想要删除的账号名称：')
        pm.delete_account(name)


def notify(title: str, message: str) -> None:
    """
    Notify
    Args:
        title (str): title of notify
        message (str): message to notify

    Returns:
    """
    applescript = f'display notification "{message}" with title "{title}"'
    # If you want to use this function, you need to install terminal-notifier
    try:
        # try to use terminal-notifier
        subprocess.call(['osascript', '-e', applescript])
    except (FileNotFoundError, TypeError):
        pass  # 忽略异常

    try:
        toaster = ToastNotifier()
        # 发送通知
        toaster.show_toast(title, message, duration=1)
    # If you want to use this function, you need to install win10toast, Mac OS does not support this function
    except (FileNotFoundError, AttributeError, TypeError):
        pass  # 忽略异常


def main() -> None:
    """
    Main Function
    Returns:

    """
    list_sign_list = ['--list', 'list', '-l']
    add_sign_list = ['add', '--add', '-a']
    help_sign_list = ['--help', '-h']
    delete_sign_list = ['--delete', 'delete', '-d']
    update_sign_list = ['--update', 'update', '-u']
    get_sign_list = ['--get', 'get', '-g']

    try:
        # If the first argument is in the list, then execute the corresponding function
        arg_1 = sys.argv[1]
        # If the first argument is in the list of list_sign_list, then print the account list
        if arg_1 in list_sign_list:
            pm.print_account_list()
        # If the first argument is in the list of help_sign_list, then print the help message
        elif arg_1 in help_sign_list:
            print_help()
        # If the first argument is in the list of add_sign_list, then add account
        elif arg_1 in add_sign_list:
            name = input(u'请输入您的要新建的账号名称(如:jenkins web root account)：')
            url = input(u'请输入您的要新建的账号的url(如:https://www.jenkins.com):')
            username = input(u'请输入你的登录用户名: ')
            password = getpass.getpass(u'请输入你的登录密码或验证token: ')
            pm.add_account(name, url, username, password, None)
        elif arg_1 in delete_sign_list:
            delete_account()
        elif arg_1 in update_sign_list:
            update_account()
        elif arg_1 in get_sign_list:
            threading.Thread(target=pm().get_password, args=(sys.argv[2],)).start()
            notify('Jax Password Manager',
                   'The content you need has been copied to the clipboard, key:{}'.format(sys.argv[2]))

        else:
            threading.Thread(target=minimize_window).start()
            threading.Thread(target=pm().get_password, args=(sys.argv[1],)).start()
            notify('Jax Password Manager',
                   'The content you need has been copied to the clipboard, key:{}'.format(sys.argv[1]))
    except IndexError:
        print_help()
    except ValueError:
        print(u'The NUM you entered is invalid, please enter a valid NUM')
    except KeyboardInterrupt:
        print(u'Quit')


if __name__ == '__main__':
    main()



# -*- coding:utf-8 -*-
"""
ask gpt
"""
import logging
import os
from jax_tools.logger import logger

import openai
from jax_tools.encrypt import AESCipher
from typing import TypeVar
import sys

_KT = TypeVar('_KT')
_VT = TypeVar('_VT')


def interact_ask_gpt() -> None:
    """
    Ask question to GPT-4
    Returns:

    """
    question_queue = [{"role": "system", "content": "你是智能AI，你叫贾克斯。"}]
    # Confirm openai key file exists
    openai_key_file = os.path.join(os.path.expanduser('~'), '.jax', '.openai.key')
    if not os.path.exists(openai_key_file):
        print('OpenAI key file not found, please create it first, use jax-encrypt -e YOUR_OPENAI_KEY '
              'to generate a encrypted key and add it to {}'.format(openai_key_file))
        exit(0)
    try:
        while True:
            question = input('请输入问题：')
            if question in ['exit', 'quit', 'q', '']:
                print("本次会话结束，再见！")
                exit(0)
            question_queue.append({"role": "user", "content": question})
            question_queue = handle_question_length(question_queue)
            logger.info('开始')
            answer = ask_gpt(question_queue)
            logger.info('结束')
            question_queue += [{"role": "assistant", "content": answer}]
            print()
    except KeyboardInterrupt:
        print("本次会话结束，再见！")
        exit(0)


def handle_question_length(question: list[_VT]) -> list[_VT]:
    """
    Handle question length
    Args:
        question (list): Question

    Returns:
        list

    """
    max_allow_length = 2200
    while True:
        if len(str(question)) > max_allow_length and len(question) > 3:
            del question[1]
        else:
            break
    return question


def ask_gpt(question_queue: list[_VT]) -> str:
    """
    Ask question to GPT-4
    Args:
        question_queue (list[_VT]): Question queue, example: [{"role": "system", "content": "你是智能AI，你叫贾克斯。
        "},{"role": "user", "content": "你好"}]

    Returns:

    """
    http_proxy = 'http://127.0.0.1:1081'
    os.environ['http_proxy'] = http_proxy
    os.environ['https_proxy'] = http_proxy
    openai_key_file = os.path.join(os.path.expanduser('~'), '.jax', '.openai.key')
    if not os.path.exists(openai_key_file):
        print('OpenAI key file not found, please create it first, the key file path is: {}'.format(openai_key_file))
    openai_key = AESCipher().decrypt(open(openai_key_file).read())
    # 设置API密钥
    openai.api_key = openai_key
    # 调用OpenAI API并使用流式处理接收响应
    response = openai.ChatCompletion.create(
        model="gpt-3.5-turbo-1106",
        messages=question_queue,
        stream=True
    )
    # Define a variable to store the response
    message = str()
    for chunk in response:
        for char in chunk['choices'][0]['delta'].get('content', ''):
            print(char, end='')
            message += char
            sys.stdout.flush()
    return message


def main() -> None:
    """
    Main
    Returns:

    """
    interact_ask_gpt()


if __name__ == '__main__':
    main()



# -*- coding:utf-8 -*-
"""
utils base
"""
from typing import TypeVar
import re

KT = TypeVar('KT')
VT = TypeVar('VT')


def sanitize_input(input_string: str) -> str:
    """
    Sanitizes an input string by escaping potentially dangerous characters.

    Args:
        input_string (str): The string to be sanitized.

    Returns:
        str: The sanitized string.
    """

    # 移除或转义特殊字符
    sanitized = re.sub(r'[;|&`\'\"*?~<>^()[\]{}$\\]', '', input_string)
    return sanitized
